# -*- coding: iso-8859-15 -*-
"""
Simple panel that defines a dialog to send APDUs to a card.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import wx

from smartcard.wx.APDUHexValidator import APDUHexValidator
from smartcard.wx.SimpleSCardAppEventObserver import \
    SimpleSCardAppEventObserver
from smartcard.util import toBytes, toHexString

[ID_TEXT_COMMAND,
 ID_TEXTCTRL_COMMAND,
 ID_TEXT_RESPONSE,
 ID_TEXTCTRL_RESPONSE,
 ID_TEXT_SW,
 ID_TEXT_SW1,
 ID_TEXTCTRL_SW1,
 ID_TEXT_SW2,
 ID_TEXTCTRL_SW2,
 ID_CARDSTATE,
 ID_TRANSMIT] = [wx.NewId() for x in range(11)]


class SampleAPDUManagerPanel(wx.Panel, SimpleSCardAppEventObserver):
    '''A simple panel that displays activated cards and readers and can
    send APDU to a connected card.'''

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        SimpleSCardAppEventObserver.__init__(self)
        self.layoutControls()

        self.Bind(wx.EVT_BUTTON, self.OnTransmit, self.transmitbutton)

    # callbacks from SimpleSCardAppEventObserver interface
    def OnActivateCard(self, card):
        """Called when a card is activated by double-clicking
        on the card or reader tree control or toolbar.
        In this sample, we just connect to the card on the first activation."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel('Activated card: ' + repr(card))
        self.transmitbutton.Enable()

    def OnActivateReader(self, reader):
        """Called when a reader is activated by double-clicking
        on the reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnActivateReader(self, reader)
        self.feedbacktext.SetLabel('Activated reader: ' + repr(reader))
        self.transmitbutton.Disable()

    def OnDeactivateCard(self, card):
        """Called when a card is deactivated in the reader
        tree control or toolbar."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel('Deactivated card: ' + repr(card))
        self.transmitbutton.Disable()

    def OnDeselectCard(self, card):
        """Called when a card is selected by clicking on the
        card or reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectCard(self, card)
        self.feedbacktext.SetLabel('Deselected card: ' + repr(card))
        self.transmitbutton.Disable()

    def OnSelectCard(self, card):
        """Called when a card is selected by clicking on the
        card or reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectCard(self, card)
        self.feedbacktext.SetLabel('Selected card: ' + repr(card))
        if hasattr(self.selectedcard, 'connection'):
            self.transmitbutton.Enable()

    def OnSelectReader(self, reader):
        """Called when a reader is selected by clicking on the
        reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectReader(self, reader)
        self.feedbacktext.SetLabel('Selected reader: ' + repr(reader))
        self.transmitbutton.Disable()

    # callbacks
    def OnTransmit(self, event):
        if hasattr(self.selectedcard, 'connection'):
            apdu = self.commandtextctrl.GetValue()
            if type(u'') == type(apdu):
                apdu = apdu.encode('utf8')
            data, sw1, sw2 = \
                self.selectedcard.connection.transmit(toBytes(apdu))
            self.SW1textctrl.SetValue("%x" % sw1)
            self.SW2textctrl.SetValue("%x" % sw2)
            self.responsetextctrl.SetValue(toHexString(data + [sw1, sw2]))
        event.Skip()

    def layoutControls(self):

        # create controls
        statictextCommand = wx.StaticText(
            self,
            ID_TEXT_COMMAND,
            "Command",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        self.commandtextctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_COMMAND, "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_MULTILINE,
            validator=APDUHexValidator())
        statictextResponse = wx.StaticText(
            self,
            ID_TEXT_RESPONSE,
            "Response",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        self.responsetextctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_RESPONSE,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_MULTILINE | wx.TE_READONLY)
        statictextStatusWords = wx.StaticText(
            self,
            ID_TEXT_SW,
            "Status Words",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        statictextSW1 = wx.StaticText(
            self,
            ID_TEXT_SW1,
            "SW1",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        self.SW1textctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_SW1,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_READONLY)
        statictextSW2 = wx.StaticText(
            self,
            ID_TEXT_SW2,
            "SW2",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        self.SW2textctrl = wx.TextCtrl(
            self,
            ID_TEXTCTRL_SW2,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            wx.TE_READONLY)
        self.feedbacktext = wx.StaticText(
            self,
            ID_CARDSTATE,
            "",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)

        # layout controls
        boxsizerCommand = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerCommand.Add(statictextCommand, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerCommand.Add(self.commandtextctrl, 5, wx.EXPAND | wx.ALL, 5)

        boxsizerResponse = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerResponse.Add(
            statictextResponse, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerResponse.Add(self.responsetextctrl, 5, wx.EXPAND | wx.ALL, 5)

        boxsizerSW = wx.BoxSizer(wx.HORIZONTAL)
        boxsizerSW.Add(statictextSW1, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerSW.Add(self.SW1textctrl, 0, wx.EXPAND | wx.ALL, 5)
        boxsizerSW.Add(statictextSW2, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        boxsizerSW.Add(self.SW2textctrl, 0, wx.EXPAND | wx.ALL, 5)

        item11 = wx.BoxSizer(wx.HORIZONTAL)
        item11.Add(statictextStatusWords, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        item11.Add(boxsizerSW, 0, wx.EXPAND | wx.ALL, 5)

        boxsizerResponseAndSW = wx.BoxSizer(wx.VERTICAL)
        boxsizerResponseAndSW.Add(boxsizerResponse, 0, wx.EXPAND | wx.ALL, 5)
        boxsizerResponseAndSW.Add(item11, 0, wx.EXPAND | wx.ALL, 5)

        staticboxAPDU = wx.StaticBox(self, -1, "APDU")
        boxsizerAPDU = wx.StaticBoxSizer(staticboxAPDU, wx.VERTICAL)
        boxsizerAPDU.Add(boxsizerCommand, 1, wx.EXPAND | wx.ALL, 5)
        boxsizerAPDU.Add(boxsizerResponseAndSW, 4, wx.EXPAND | wx.ALL, 5)

        staticboxEvents = wx.StaticBox(self, -1, "Card/Reader Events")
        boxsizerEvents = wx.StaticBoxSizer(staticboxEvents, wx.HORIZONTAL)
        boxsizerEvents.Add(self.feedbacktext, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        sizerboxTransmitButton = wx.BoxSizer(wx.HORIZONTAL)
        sizerboxTransmitButton.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        self.transmitbutton = wx.Button(
            self,
            ID_TRANSMIT,
            "Transmit",
            wx.DefaultPosition,
            wx.DefaultSize,
            0)
        self.transmitbutton.Disable()
        sizerboxTransmitButton.Add(
            self.transmitbutton, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizerboxTransmitButton.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)

        sizerPanel = wx.BoxSizer(wx.VERTICAL)
        sizerPanel.Add(boxsizerAPDU, 3, wx.EXPAND | wx.ALL, 5)
        sizerPanel.Add(boxsizerEvents, 1, wx.EXPAND | wx.ALL, 5)
        sizerPanel.Add(sizerboxTransmitButton, 1, wx.EXPAND | wx.ALL, 5)

        self.SetSizer(sizerPanel)
        self.SetAutoLayout(True)
        sizerPanel.Fit(self)



#! /usr/bin/env python
"""
Sample script that tries to select the DF_TELECOM on all inserted cards.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.CardType import AnyCardType
from smartcard.CardRequest import CardRequest
from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.Exceptions import CardRequestTimeoutException
import sys

# define the apdus used in this script
GET_RESPONSE = [0XA0, 0XC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]

# request any card type
cardtype = AnyCardType()

try:
    # request card insertion
    print('insert a card (SIM card if possible) within 10s')
    cardrequest = CardRequest(timeout=10, cardType=cardtype)
    cardservice = cardrequest.waitforcard()

    # attach the console tracer
    observer = ConsoleCardConnectionObserver()
    cardservice.connection.addObserver(observer)

    # connect to the card and perform a few transmits
    cardservice.connection.connect()

    apdu = SELECT + DF_TELECOM
    response, sw1, sw2 = cardservice.connection.transmit(apdu)

    # there is a DF_TELECOM
    if sw1 == 0x9F:
        apdu = GET_RESPONSE + [sw2]
        response, sw1, sw2 = cardservice.connection.transmit(apdu)

    else:
        print('no DF_TELECOM')

except CardRequestTimeoutException:
    print('time-out: no card inserted during last 10s')

if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



"""Smartcard system utility functions and classes.

Manages smartcard readers and reader groups.

__author__ = "gemalto http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import smartcard.reader.ReaderFactory
import smartcard.pcsc.PCSCReaderGroups


def readers(groups=[]):
    """Returns the list of smartcard readers in groups.

    If group is not specified, returns the list of all smartcard readers.

    import smartcard
    r=smartcard.readers()
    r=smartcard.readers(['SCard$DefaultReaders', 'MyReaderGroup'])
    """

    return smartcard.reader.ReaderFactory.ReaderFactory.readers(groups)


def readergroups():
    """Returns the list of reader groups."""

    return smartcard.pcsc.PCSCReaderGroups.PCSCReaderGroups().instance


# for legacy only
def listReaders():
    """Returns the list of smartcard readers.

    Deprecated - Use smartcard.System.readers() instead.
    """
    zreaders = []
    for reader in readers():
        zreaders.append(str(reader))
    return zreaders


if __name__ == '__main__':
    print(readers())
    print(readers(['SCard$DefaultReaders']))
    print(readergroups())



#! /usr/bin/env python
"""Unit tests for smartcard.readers.

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
import unittest

# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

try:
    from local_config import expectedATRs, expectedReaders
    from local_config import expectedReaderGroups, expectedATRinReader
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


from smartcard.System import readers, readergroups
from smartcard import listReaders
from smartcard.scard import resourceManager


class testcase_readers(unittest.TestCase):
    """Test smartcard framework readers factory methods"""

    def testcase_enoughreaders(self):
        self.assertTrue(len(readers()) > 1)

    def testcase_readers(self):
        foundreaders = {}
        for reader in readers():
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_hashreaders(self):
        foundreaders = {}
        for reader in readers():
            foundreaders[reader] = 1
        for reader in list(foundreaders.keys()):
            self.assertTrue(reader in readers())

    def testcase_legacyreaders(self):
        foundreaders = {}
        for reader in listReaders():
            foundreaders[reader] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_readers_in_readergroup(self):
        foundreaders = {}
        for reader in readers(['SCard$DefaultReaders']):
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    def testcase_readers_in_readergroup_empty(self):
        foundreaders = {}
        for reader in readers([]):
            foundreaders[str(reader)] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundreaders)

    if 'winscard' == resourceManager:

        def testcase_readers_in_readergroup_nonexistent(self):
            foundreaders = {}
            for reader in readers(['dummy$group']):
                foundreaders[reader] = 1
            for reader in expectedReaders:
                self.assertTrue(not reader in foundreaders)
            self.assertEqual(0, len(foundreaders))

    def testcase_readergroups(self):
        foundreadergroups = {}
        for readergroup in readergroups():
            foundreadergroups[readergroup] = 1
        for readergroup in expectedReaderGroups:
            self.assertTrue(readergroup in foundreadergroups)


def suite():
    suite1 = unittest.makeSuite(testcase_readers)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for smartcard.ReaderMonitoring

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
import threading
import unittest
import time


# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

try:
    from local_config import expectedATRs, expectedReaders
    from local_config import expectedReaderGroups, expectedATRinReader
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


from smartcard.ReaderMonitoring import ReaderMonitor, ReaderObserver


# a simple reader observer that prints added/removed readers
class printobserver(ReaderObserver):

    def __init__(self, obsindex, testcase):
        self.obsindex = obsindex
        self.testcase = testcase

    def update(self, observable, actions):
        (addedreaders, removedreaders) = actions
        foundreaders = {}
        self.testcase.assertEqual(removedreaders, [])
        for reader in addedreaders:
            foundreaders[str(reader)] = 1
        if {} != foundreaders:
            for reader in expectedReaders:
                self.testcase.assertTrue(reader in foundreaders)


class testthread(threading.Thread):

    def __init__(self, obsindex, testcase):
        threading.Thread.__init__(self)
        self.obsindex = obsindex
        self.testcase = testcase
        self.readermonitor = ReaderMonitor()
        self.observer = None

    def run(self):
        # create and register observer
        self.observer = printobserver(self.obsindex, self.testcase)
        self.readermonitor.addObserver(self.observer)
        time.sleep(1)
        self.readermonitor.deleteObserver(self.observer)


class testcase_readermonitor(unittest.TestCase):
    """Test smartcard framework reader monitoring methods"""

    def testcase_readermonitorthread(self):
        threads = []
        for i in range(0, 4):
            t = testthread(i, self)
            threads.append(t)
        for t in threads:
            t.start()
        for t in threads:
            t.join()


def suite():
    suite1 = unittest.makeSuite(testcase_readermonitorthread)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



"""Smartcard module exceptions.

This module defines the exceptions raised by the smartcard module.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


class SmartcardException(Exception):
    """Base class for smartcard exceptions.

    smartcard exceptions are generated by the smartcard module and
    shield scard (i.e. PCSC) exceptions raised by the scard module.

    """
    pass


class CardConnectionException(SmartcardException):
    """Raised when a CardConnection class method fails."""
    pass


class CardRequestException(SmartcardException):
    """Raised when a CardRequest wait fails."""
    pass


class CardRequestTimeoutException(SmartcardException):
    """Raised when a CardRequest times out."""

    def __init__(self, *args):
        SmartcardException.__init__(self,
                                    "Time-out during card request", *args)


class CardServiceException(SmartcardException):
    """Raised when a CardService class method fails."""
    pass


class InvalidATRMaskLengthException(SmartcardException):
    """Raised when an ATR mask does not match an ATR length."""

    def __init__(self, *args):
        SmartcardException.__init__(self,
                                    'Invalid ATR mask length: ', *args)


class InvalidReaderException(SmartcardException):
    """Raised when trying to acces an invalid smartcard reader."""

    def __init__(self, *args):
        SmartcardException.__init__(self, 'Invalid reader: ', *args)


class ListReadersException(SmartcardException):
    """Raised when smartcard readers cannot be listed."""

    def __init__(self, *args):
        SmartcardException.__init__(self, 'Failed to list readers', *args)


class NoCardException(SmartcardException):
    """Raised when no card in is present in reader."""

    def __init__(self, *args):
        SmartcardException.__init__(
            self,
            'Unable to connect to card or no card in reader',
            *args)


class NoReadersException(SmartcardException):
    """Raised when the system has no smartcard reader."""

    def __init__(self, *args):
        SmartcardException.__init__(self, 'No reader found', *args)



"""Base class for status word error checkers.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


class ErrorChecker(object):
    """Base class for status word error checking strategies.

    Error checking strategies are chained into an L{ErrorCheckingChain} to
    implement a Chain of Responsibility. Each strategy in the chain is
    called until an error is  detected. The strategy raises a
    L{smartcard.sw.SWException} exception when an error is detected.

    Implementation derived from Bruce Eckel, Thinking in Python. The
    L{ErrorCheckingChain} implements the Chain Of Responsibility design
    pattern.
    """

    def __call__(data, sw1, sw2):
        """Called to test data, sw1 and sw2 for error.

        @param data:       apdu response data
        @param sw1, sw2:   apdu data status words

        Derived classes must raise a L{smartcard.sw.SWException} upon error."""
        pass



#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: Locate cards in the system

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.scard import *
import sys

if 'winscard' == resourceManager:

    znewcardName = 'dummy-card'
    znewcardATR = [0x3B, 0x77, 0x94, 0x00, 0x00, 0x82, 0x30, 0x00, 0x13,
                   0x6C, 0x9F, 0x22]
    znewcardMask = [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                    0xFF, 0xFF, 0xFF]

    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise scard.error(
                'Failed to establish context: ' +
                SCardGetErrorMessage(hresult))
        print('Context established!')

        try:
            hresult, readers = SCardListReaders(hcontext, [])
            if hresult != SCARD_S_SUCCESS:
                raise scard.error(
                    'Failed to list readers: ' +
                    SCardGetErrorMessage(hresult))
            print('PCSC Readers:', readers)

            # introduce a card (forget first in case it is already present)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            print('Introducing card ' + znewcardName)
            hresult = SCardIntroduceCardType(hcontext, znewcardName, [],
                                             [], znewcardATR, znewcardMask)
            if hresult != SCARD_S_SUCCESS:
                if hresult == ERROR_ALREADY_EXISTS:
                    print('Card already exists')
                else:
                    raise error(
                        'Failed to introduce card type: ' +
                        SCardGetErrorMessage(hresult))

            hresult, cards = SCardListCards(hcontext, [], [])
            if hresult != SCARD_S_SUCCESS:
                raise error('Failure to list cards')
            print('Cards:', cards)

            readerstates = []
            for i in range(len(readers)):
                readerstates += [(readers[i], SCARD_STATE_UNAWARE)]
            print(readerstates)

            hresult, newstates = SCardLocateCards(
                hcontext,
                cards,
                readerstates)
            for i in newstates:
                reader, eventstate, atr = i
                print(reader, end=' ')
                for b in atr:
                    print("0x%.2X" % b, end=' ')
                print("")
                if eventstate & SCARD_STATE_ATRMATCH:
                    print('Card found')
                if eventstate & SCARD_STATE_UNAWARE:
                    print('State unware')
                if eventstate & SCARD_STATE_IGNORE:
                    print('Ignore reader')
                if eventstate & SCARD_STATE_UNAVAILABLE:
                    print('Reader unavailable')
                if eventstate & SCARD_STATE_EMPTY:
                    print('Reader empty')
                if eventstate & SCARD_STATE_PRESENT:
                    print('Card present in reader')
                if eventstate & SCARD_STATE_EXCLUSIVE:
                    print('Card allocated for exclusive use')
                if eventstate & SCARD_STATE_INUSE:
                    print('Card in use but can be shared')
                if eventstate & SCARD_STATE_MUTE:
                    print('Card is mute')
                if eventstate & SCARD_STATE_CHANGED:
                    print('State changed')
                if eventstate & SCARD_STATE_UNKNOWN:
                    print('State unknowned')

        finally:
            hresult = SCardForgetCardType(hcontext, znewcardName)
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failed to release context: ' +
                    SCardGetErrorMessage(hresult))
            print('Released context.')

    except error as e:
        print(e)

elif 'pcsclite' == resourceManager:
    print('SCardLocateCards not supported by pcsc lite')


if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
"""Unit tests for ulist

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import unittest

from smartcard.ulist import ulist


class C(ulist):

    def __onadditem__(self, item):
        #print '+', item
        pass

    def __onremoveitem__(self, item):
        #print '-', item
        pass


class testcase_ulist(unittest.TestCase):
    """Test smartcard.ulist."""

    def testcase_ulist_init(self):
        """tests constructor"""

        c = C([1, 2, 3, 3, 4, 5, 5])
        self.assertEqual([1, 2, 3, 4, 5], c)

        c = C(['one', 'two', 'three', 'one'])
        self.assertEqual(['one', 'two', 'three'], c)

    def testcase_ulist_add(self):
        """tests l=l+other"""

        seed = [1, 2, 3]
        c = C(seed)
        self.assertEqual(seed, c)

        c = c + []
        self.assertEqual(seed, c)

        c = c + 4
        self.assertEqual(seed + [4], c)

        c = c + 4
        self.assertEqual(seed + [4], c)

        c = c + 'word'
        self.assertEqual(seed + [4] + ['word'], c)

        seed = ['one', 'two', 'three']
        c = C(seed)
        self.assertEqual(seed, c)

        c = c + ['four', 'five']
        self.assertEqual(seed + ['four', 'five'], c)

    def testcase_ulist_iadd(self):
        """tests l+=other"""

        seed = [1, 2, 3]
        c = C(seed)
        self.assertEqual(seed, c)

        c += []
        self.assertEqual(seed, c)

        c += 4
        self.assertEqual(seed + [4], c)

        c += 4
        self.assertEqual(seed + [4], c)

        c += [4, 3, 2, 1]
        self.assertEqual(seed + [4], c)

        c += 'word'
        self.assertEqual(seed + [4] + ['word'], c)

        seed = ['one', 'two', 'three']
        c = C(seed)
        self.assertEqual(seed, c)

        c += ['four', 'five']
        self.assertEqual(seed + ['four', 'five'], c)

    def testcase_ulist_radd(self):
        """tests l=other+l"""

        seed = [1, 2, 3]
        c = C(seed)
        self.assertEqual(seed, c)

        l = [] + c
        self.assertEqual(seed, l)

        l = [3] + c
        self.assertEqual(seed, c)
        self.assertEqual(seed, l)

        l = [3, 3, 4, 4] + c
        self.assertEqual(seed, c)
        self.assertEqual(seed + [4], l)

        l = [4] + ['word'] + c
        self.assertEqual(seed, c)
        self.assertEqual(seed + [4] + ['word'], l)

    def testcase_ulist_append(self):

        seed = [1, 2, 3]
        c = C(seed)

        c.append(4)
        self.assertEqual(seed + [4], c)

        c.append(4)
        self.assertEqual(seed + [4], c)

        c.append('word')
        self.assertEqual(seed + [4] + ['word'], c)

    def testcase_ulist_insert(self):

        seed = [1, 2, 3]
        c = C(seed)

        c.insert(0, 0)
        self.assertEqual([0] + seed, c)

        c.insert(1, 0)
        self.assertEqual([0] + seed, c)

    def testcase_ulist_pop(self):

        seed = [1, 2, 3]
        c = C(seed)

        c.pop()
        self.assertEqual(c, [1, 2])

        c.pop(1)
        self.assertEqual(c, [1])

    def testcase_ulist_remove(self):

        seed = [1, 2, 3]
        c = C(seed)

        c.remove(2)
        self.assertEqual(c, [1, 3])

        c.remove(1)
        self.assertEqual(c, [3])


def suite():
    suite1 = unittest.makeSuite(testcase_ulist)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for smartcard.sw

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
import unittest
import string

# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

from smartcard.sw.ErrorCheckingChain import ErrorCheckingChain
from smartcard.sw.ErrorChecker import ErrorChecker
from smartcard.sw.ISO7816_4ErrorChecker import ISO7816_4ErrorChecker
from smartcard.sw.ISO7816_4_SW1ErrorChecker import ISO7816_4_SW1ErrorChecker
from smartcard.sw.ISO7816_8ErrorChecker import ISO7816_8ErrorChecker
from smartcard.sw.ISO7816_9ErrorChecker import ISO7816_9ErrorChecker
from smartcard.sw.op21_ErrorChecker import op21_ErrorChecker
import smartcard.sw.SWExceptions


try:
    from local_config import expectedATRs, expectedReaders
    from local_config import expectedReaderGroups, expectedATRinReader
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


class CustomSWException(smartcard.sw.SWExceptions.SWException):
    """Test exception raised by TestErrorChecker."""

    def __init__(self, data, sw1, sw2, message=""):
        smartcard.sw.SWExceptions.SWException.__init__(self, data, sw1, sw2)


class TestErrorChecker(ErrorChecker):
    """Test error checking checker.

    This checker raises the following exception:

    sw1: 56 sw2: 55      CustomSWException
    sw1: 63 sw2: any     WarningProcessingException
    """

    def __call__(self, data, sw1, sw2):
        if 0x56 == sw1 and 0x55 == sw2:
            raise CustomSWException(data, sw1, sw2)
        elif 0x63 == sw1:
            raise CustomSWException(data, sw1, sw2)


class testcase_ErrorChecking(unittest.TestCase):
    """Test case for smartcard.sw.* error checking."""

    def failUnlessRaises(self, excClass, callableObj, *args, **kwargs):
        """override of unittest.TestCase.failUnlesRaises so that we return the
           exception object for testing fields."""
        try:
            callableObj(*args, **kwargs)
        except excClass as e:
            return e
        else:
            if hasattr(excClass, '__name__'):
                excName = excClass.__name__
            else:
                excName = str(excClass)
            raise self.failureException(excName)

    def testcase_ISO7816_4SW1ErrorChecker(self):
        """Test ISO7816_4_SW1ErrorChecker."""
        ecs = ISO7816_4_SW1ErrorChecker()

        tiso7816_4SW1 = {
            0x62: smartcard.sw.SWExceptions.WarningProcessingException,
            0x63: smartcard.sw.SWExceptions.WarningProcessingException,
            0x64: smartcard.sw.SWExceptions.ExecutionErrorException,
            0x65: smartcard.sw.SWExceptions.ExecutionErrorException,
            0x66: smartcard.sw.SWExceptions.SecurityRelatedException,
            0x67: smartcard.sw.SWExceptions.CheckingErrorException,
            0x68: smartcard.sw.SWExceptions.CheckingErrorException,
            0x69: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6A: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6B: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6C: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6D: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6E: smartcard.sw.SWExceptions.CheckingErrorException,
            0x6F: smartcard.sw.SWExceptions.CheckingErrorException,
        }

        for sw1 in range(0x00, 0xff + 1):
            sw2range = []
            exception = None
            if sw1 in tiso7816_4SW1:
                exception = tiso7816_4SW1[sw1]
            for sw2 in range(0x00, 0xff + 1):
                if None != exception:
                    self.assertRaises(exception, ecs, [], sw1, sw2)
                else:
                    ecs([], sw1, sw2)

    def testcase_ISO7816_4ErrorChecker(self):
        """Test ISO7816_4ErrorChecker."""
        ecs = ISO7816_4ErrorChecker()

        tiso7816_4SW = {
            0x62: (smartcard.sw.SWExceptions.WarningProcessingException,
                    [0x00, 0x81, 0x82, 0x83, 0x84, 0xff]),
            0x63: (smartcard.sw.SWExceptions.WarningProcessingException,
                    [0x00, 0x81] + list(range(0xc0, 0xcf + 1))),
            0x64: (smartcard.sw.SWExceptions.ExecutionErrorException,
                    [0x00]),
            0x67: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x00]),
            0x68: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x81, 0x82]),
            0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
                    list(range(0x81, 0x88 + 1))),
            0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
                    list(range(0x80, 0x88 + 1))),
            0x6B: (smartcard.sw.SWExceptions.CheckingErrorException, [0x00]),
            0x6D: (smartcard.sw.SWExceptions.CheckingErrorException, [0x00]),
            0x6E: (smartcard.sw.SWExceptions.CheckingErrorException, [0x00]),
            0x6F: (smartcard.sw.SWExceptions.CheckingErrorException, [0x00]),
        }

        for sw1 in range(0x00, 0xff + 1):
            sw2range = []
            if sw1 in tiso7816_4SW:
                exception, sw2range = tiso7816_4SW[sw1]
            for sw2 in range(0x00, 0xff + 1):
                if sw2 in sw2range:
                    self.assertRaises(exception, ecs, [], sw1, sw2)
                else:
                    ecs([], sw1, sw2)

    def testcase_ISO7816_8ErrorChecker(self):
        """Test ISO7816_4ErrorChecker."""
        ecs = ISO7816_8ErrorChecker()

        tiso7816_8SW = {
            0x63: (smartcard.sw.SWExceptions.WarningProcessingException,
                    [0x00] + list(range(0xc0, 0xcf + 1))),
            0x65: (smartcard.sw.SWExceptions.ExecutionErrorException,
                    [0x81]),
            0x66: (smartcard.sw.SWExceptions.SecurityRelatedException,
                    [0x00, 0x87, 0x88]),
            0x67: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x00]),
            0x68: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x83, 0x84]),
            0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
                    list(range(0x82, 0x85 + 1))),
            0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x81, 0x82, 0x86, 0x88]),
        }

        for sw1 in range(0x00, 0xff + 1):
            sw2range = []
            if sw1 in tiso7816_8SW:
                exception, sw2range = tiso7816_8SW[sw1]
            for sw2 in range(0x00, 0xff + 1):
                if sw2 in sw2range:
                    self.assertRaises(exception, ecs, [], sw1, sw2)
                else:
                    ecs([], sw1, sw2)

    def testcase_ISO7816_9ErrorChecker(self):
        """Test ISO7816_4ErrorChecker."""
        ecs = ISO7816_9ErrorChecker()

        tiso7816_9SW = {
            0x62: (smartcard.sw.SWExceptions.WarningProcessingException,
                    [0x82]),
            0x64: (smartcard.sw.SWExceptions.ExecutionErrorException,
                    [0x00]),
            0x69: (smartcard.sw.SWExceptions.CheckingErrorException, [
                    0x82]),
            0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x80, 0x84, 0x89, 0x8A]),
        }

        for sw1 in range(0x00, 0xff + 1):
            sw2range = []
            if sw1 in tiso7816_9SW:
                exception, sw2range = tiso7816_9SW[sw1]
            for sw2 in range(0x00, 0xff + 1):
                if sw2 in sw2range:
                    self.assertRaises(exception, ecs, [], sw1, sw2)
                else:
                    ecs([], sw1, sw2)

    def testcase_op21_ErrorChecker(self):
        """Test op21_ErrorChecker."""
        ecs = op21_ErrorChecker()

        top21_SW = {
            0x62: (smartcard.sw.SWExceptions.WarningProcessingException, [
                    0x83]),
            0x63: (smartcard.sw.SWExceptions.WarningProcessingException,
                    [0x00]),
            0x64: (smartcard.sw.SWExceptions.ExecutionErrorException, [
                    0x00]),
            0x65: (smartcard.sw.SWExceptions.ExecutionErrorException, [
                    0x81]),
            0x67: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x00]),
            0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x82, 0x85]),
            0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x80, 0x81, 0x82, 0x84, 0x86, 0x88]),
            0x6D: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x00]),
            0x6E: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x00]),
            0x94: (smartcard.sw.SWExceptions.CheckingErrorException,
                    [0x84, 0x85]),
        }

        for sw1 in range(0x00, 0xff + 1):
            sw2range = []
            if sw1 in top21_SW:
                exception, sw2range = top21_SW[sw1]
            for sw2 in range(0x00, 0xff + 1):
                if sw2 in sw2range:
                    self.assertRaises(exception, ecs, [], sw1, sw2)
                else:
                    ecs([], sw1, sw2)

    def testcase_ISO78164_Test_ErrorCheckingChain(self):
        """Test error chain with ISO7816-4 checker followed by Test checker."""
        errorchain = []
        errorchain = [ErrorCheckingChain(errorchain, ISO7816_4ErrorChecker()),
                     ErrorCheckingChain(errorchain, TestErrorChecker())]

        # ISO7816-4 is answering first on the next, i.e
        # WarningProcessingException
        for sw2 in [0x00, 0x81] + list(range(0xc0, 0xcf + 1)):
            self.assertRaises(
                smartcard.sw.SWExceptions.WarningProcessingException,
                errorchain[0], [], 0x63, sw2)

    def testcase_Test_ISO78164_ErrorCheckingChain(self):
        """Test error chain with Test checker followed by ISO7816-4 checker."""
        errorchain = []
        errorchain = [ErrorCheckingChain(errorchain, TestErrorChecker()),
                     ErrorCheckingChain(errorchain, ISO7816_4ErrorChecker())]

        # TestErrorChecker is answering first, i.e. CustomSWException
        for sw2 in [0x00, 0x81] + list(range(0xc0, 0xcf + 1)):
            self.assertRaises(CustomSWException, errorchain[0], [], 0x63, sw2)

    def testcase_ErrorMessage(self):
        """Test correct exception error message."""
        ecs = ISO7816_4ErrorChecker()

        e = self.failUnlessRaises(
            smartcard.sw.SWExceptions.CheckingErrorException,
            ecs,
            [],
            0x69,
            0x85)
        self.assertEqual(
            str(e),
            "'Status word exception: checking error - " + \
             "Conditions of use not satisfied!'")

        e = self.failUnlessRaises(
            smartcard.sw.SWExceptions.CheckingErrorException,
            ecs,
            [],
            0x6b,
            0x00)
        self.assertEqual(
            str(e), "'Status word exception: checking error - " + \
            "Incorrect parameters P1-P2!'")

    def testcase_ISO78164_Test_ErrorCheckingChain_filtering(self):
        """Test error chain with ISO7816-4 checker followed by Test checker."""
        errorchain = []
        errorchain = [ErrorCheckingChain(errorchain, ISO7816_8ErrorChecker()),
                     ErrorCheckingChain(errorchain, ISO7816_4ErrorChecker()),
                     ErrorCheckingChain(
                         errorchain, ISO7816_4_SW1ErrorChecker())]

        # don't care about Warning Exceptions
        errorchain[0].addFilterException(
            smartcard.sw.SWExceptions.WarningProcessingException)

        for sw2 in range(0x00, 0xff):

            # should not raise
            errorchain[0]([], 0x62, sw2)
            errorchain[0]([], 0x63, sw2)
            # should raise
            self.assertRaises(
                smartcard.sw.SWExceptions.ExecutionErrorException,
                errorchain[0],
                [],
                0x64,
                sw2)


def suite():
    suite1 = unittest.makeSuite(testcase_ErrorChecking)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""
Sample script that demonstrates how to create a custom CardType.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.CardType import CardType
from smartcard.CardRequest import CardRequest
from smartcard.util import toHexString


class DCCardType(CardType):
    # define our custom CardType
    # this card type defines direct convention card
    # (first atr byte equal to 0x3b)

    def matches(self, atr, reader=None):
        return atr[0] == 0x3B


# request a direct convention card
cardtype = DCCardType()
cardrequest = CardRequest(timeout=1, cardType=cardtype)
cardservice = cardrequest.waitforcard()


# connect and print atr and reader
cardservice.connection.connect()
print(toHexString(cardservice.connection.getATR()))
print(cardservice.connection.getReader())


import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



"""CardConnectionObserver interface.

CardConnectionObserver is a base class for objects that are to be notified
upon CardConnection events.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.util import toHexString
from smartcard.Observer import Observer


# ReaderObserver interface
class CardConnectionObserver(Observer):
    """
    CardConnectionObserver is a base class for objects that are to be notified
    upon CardConnection events.
    """

    def update(self, cardconnection, cardconnectionevent):
        """Called upon CardConnection event.

        cardconnection:         the observed card connection object
        cardconnectionevent:    the CardConnectionEvent sent by the connection
        """
        pass


class ConsoleCardConnectionObserver(CardConnectionObserver):

    def update(self, cardconnection, ccevent):

        if 'connect' == ccevent.type:
            print('connecting to ' + cardconnection.getReader())

        elif 'disconnect' == ccevent.type:
            print('disconnecting from ' + cardconnection.getReader())

        elif 'command' == ccevent.type:
            print('> ' + toHexString(ccevent.args[0]))

        elif 'response' == ccevent.type:
            if [] == ccevent.args[0]:
                print('<  [] %02X %02X' % tuple(ccevent.args[-2:]))
            else:
                print('< ' +
                      toHexString(ccevent.args[0]) + " " +
                      "%02X %02X" % tuple(ccevent.args[-2:]))



#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: List card attributes

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import struct
from smartcard.scard import *
import smartcard.util
import sys

attributes = {
    SCARD_ATTR_ATR_STRING: 'SCARD_ATTR_ATR_STRING',
    SCARD_ATTR_CHANNEL_ID: 'SCARD_ATTR_CHANNEL_ID',
    SCARD_ATTR_CHARACTERISTICS: 'SCARD_ATTR_CHARACTERISTICS',
    SCARD_ATTR_CURRENT_BWT: 'SCARD_ATTR_CURRENT_BWT',
    SCARD_ATTR_CURRENT_CLK: 'SCARD_ATTR_CURRENT_CLK',
    SCARD_ATTR_CURRENT_CWT: 'SCARD_ATTR_CURRENT_CWT',
    SCARD_ATTR_CURRENT_D: 'SCARD_ATTR_CURRENT_D',
    SCARD_ATTR_CURRENT_EBC_ENCODING: 'SCARD_ATTR_CURRENT_EBC_ENCODING',
    SCARD_ATTR_CURRENT_F: 'SCARD_ATTR_CURRENT_F',
    SCARD_ATTR_CURRENT_IFSC: 'SCARD_ATTR_CURRENT_IFSC',
    SCARD_ATTR_CURRENT_IFSD: 'SCARD_ATTR_CURRENT_IFSD',
    SCARD_ATTR_CURRENT_IO_STATE: 'SCARD_ATTR_CURRENT_IO_STATE',
    SCARD_ATTR_CURRENT_N: 'SCARD_ATTR_CURRENT_N',
    SCARD_ATTR_CURRENT_PROTOCOL_TYPE: 'SCARD_ATTR_CURRENT_PROTOCOL_TYPE',
    SCARD_ATTR_CURRENT_W: 'SCARD_ATTR_CURRENT_W',
    SCARD_ATTR_DEFAULT_CLK: 'SCARD_ATTR_DEFAULT_CLK',
    SCARD_ATTR_DEFAULT_DATA_RATE: 'SCARD_ATTR_DEFAULT_DATA_RATE',
    SCARD_ATTR_DEVICE_FRIENDLY_NAME_A: 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_A',
    SCARD_ATTR_DEVICE_FRIENDLY_NAME_W: 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_W',
    SCARD_ATTR_DEVICE_IN_USE: 'SCARD_ATTR_DEVICE_IN_USE',
    SCARD_ATTR_DEVICE_SYSTEM_NAME_A: 'SCARD_ATTR_DEVICE_SYSTEM_NAME_A',
    SCARD_ATTR_DEVICE_SYSTEM_NAME_W: 'SCARD_ATTR_DEVICE_SYSTEM_NAME_W',
    SCARD_ATTR_DEVICE_UNIT: 'SCARD_ATTR_DEVICE_UNIT',
    SCARD_ATTR_ESC_AUTHREQUEST: 'SCARD_ATTR_ESC_AUTHREQUEST',
    SCARD_ATTR_ESC_CANCEL: 'SCARD_ATTR_ESC_CANCEL',
    SCARD_ATTR_ESC_RESET: 'SCARD_ATTR_ESC_RESET',
    SCARD_ATTR_EXTENDED_BWT: 'SCARD_ATTR_EXTENDED_BWT',
    SCARD_ATTR_ICC_INTERFACE_STATUS: 'SCARD_ATTR_ICC_INTERFACE_STATUS',
    SCARD_ATTR_ICC_PRESENCE: 'SCARD_ATTR_ICC_PRESENCE',
    SCARD_ATTR_ICC_TYPE_PER_ATR: 'SCARD_ATTR_ICC_TYPE_PER_ATR',
    SCARD_ATTR_MAXINPUT: 'SCARD_ATTR_MAXINPUT',
    SCARD_ATTR_MAX_CLK: 'SCARD_ATTR_MAX_CLK',
    SCARD_ATTR_MAX_DATA_RATE: 'SCARD_ATTR_MAX_DATA_RATE',
    SCARD_ATTR_MAX_IFSD: 'SCARD_ATTR_MAX_IFSD',
    SCARD_ATTR_POWER_MGMT_SUPPORT: 'SCARD_ATTR_POWER_MGMT_SUPPORT',
    SCARD_ATTR_SUPRESS_T1_IFS_REQUEST: 'SCARD_ATTR_SUPRESS_T1_IFS_REQUEST',
    SCARD_ATTR_USER_AUTH_INPUT_DEVICE: 'SCARD_ATTR_USER_AUTH_INPUT_DEVICE',
    SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE:
        'SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE',
    SCARD_ATTR_VENDOR_IFD_SERIAL_NO: 'SCARD_ATTR_VENDOR_IFD_SERIAL_NO',
    SCARD_ATTR_VENDOR_IFD_TYPE: 'SCARD_ATTR_VENDOR_IFD_TYPE',
    SCARD_ATTR_VENDOR_IFD_VERSION: 'SCARD_ATTR_VENDOR_IFD_VERSION',
    SCARD_ATTR_VENDOR_NAME: 'SCARD_ATTR_VENDOR_NAME',
    }
if 'pcsclite' == resourceManager:
    extra_attributes = {
        SCARD_ATTR_ASYNC_PROTOCOL_TYPES: 'SCARD_ATTR_ASYNC_PROTOCOL_TYPES',
        SCARD_ATTR_SYNC_PROTOCOL_TYPES: 'SCARD_ATTR_SYNC_PROTOCOL_TYPES',
        }
    attributes.update(extra_attributes)


def printAttribute(attrib, value):
    print('-----------------', attributes[attrib], '-----------------')
    print(value)
    print(smartcard.util.toHexString(value, smartcard.util.HEX))
    print(struct.pack(*['<' + 'B' * len(value)] + value))


if __name__ == '__main__':
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error(
            'Failed to establish context: ' +
            SCardGetErrorMessage(hresult))
    print('Context established!')

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to list readers: ' +
                SCardGetErrorMessage(hresult))
        print('PCSC Readers:', readers)

        if len(readers) < 1:
            raise error('No smart card readers')

        for reader in readers:
            print('Trying to retreive attributes of', reader)
            hresult, hcard, dwActiveProtocol = SCardConnect(
                hcontext,
                reader,
                SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1)
            if hresult != SCARD_S_SUCCESS:
                print(error,
                      'Unable to connect: ' + SCardGetErrorMessage(hresult))
            else:

                print('Connected with active protocol', dwActiveProtocol)

                try:
                    for i in list(attributes.keys()):
                        hresult, attrib = SCardGetAttrib(hcard, i)
                        if hresult == SCARD_S_SUCCESS:
                            printAttribute(i, attrib)
                        else:
                            print('-----------------',
                                  attributes[i],
                                  '-----------------')
                            print('error:', SCardGetErrorMessage(hresult))

                finally:
                    hresult = SCardDisconnect(hcard, SCARD_UNPOWER_CARD)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            'Failed to disconnect: ' +
                            SCardGetErrorMessage(hresult))
                    print('Disconnected')

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to release context: ' +
                SCardGetErrorMessage(hresult))
        print('Released context.')

if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
"""Unit tests for SCardIntroduceCardType/SCardListCards/SCardListInterfaces

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import unittest
import platform
from smartcard.scard import *
import smartcard.guid

if 'winscard' == resourceManager:

    class testcase_listcards(unittest.TestCase):
        """Test scard API for ATR retrieval"""

        # setup for all unit tests: establish context and introduce
        # a dummy card interface
        def setUp(self):
            hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
            self.assertEqual(hresult, 0)
            self.dummycardname = 'dummycard'
            self.dummycardATR = \
                [0x3B, 0x75, 0x94, 0x00, 0x00, 0x62, 0x02, 0x02, 0x01, 0x01]
            self.dummycardMask = \
                [0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
            self.dummycardguid1 = \
                smartcard.guid.strToGUID(
                    '{AD4F1667-EA75-4124-84D4-641B3B197C65}')
            self.dummycardguid2 = \
                smartcard.guid.strToGUID(
                    '{382AE95A-7C2C-449c-A179-56C6DE6FF3BC}')
            testcase_listcards.__introduceinterface(self)

        # teardown for all unit tests: release context and forget
        # dummy card interface
        def tearDown(self):
            testcase_listcards.__forgetinterface(self)
            hresult = SCardReleaseContext(self.hcontext)
            self.assertEqual(hresult, 0)

        # introduce a dummy card interface
        # card ATR same as e-gate
        def __introduceinterface(self):
            hresult = SCardForgetCardType(self.hcontext, self.dummycardname)
            dummycardPrimaryGUID = self.dummycardguid1
            dummycardGUIDS = self.dummycardguid1 + self.dummycardguid2
            hresult = SCardIntroduceCardType(
                        self.hcontext,
                        self.dummycardname,
                        dummycardPrimaryGUID,
                        dummycardGUIDS,
                        self.dummycardATR,
                        self.dummycardMask)
            self.assertEqual(hresult, 0)

        # forget dummy card interface
        def __forgetinterface(self):
            hresult = SCardForgetCardType(self.hcontext, self.dummycardname)
            self.assertEqual(hresult, 0)

        # locate a known card
        # Cryptoflex 8k v2 is present in standard Windows 2000
        def test_listcryptoflexbyatr(self):
            if -1 != platform.platform().find('Windows-7'):
                dmyATR = \
                  [0x3B, 0x75, 0x94, 0x00, 0x00, 0x62, 0x02, 0x02, 0x01, 0x01]
                dmyName = ['dummycard']
                hresult, card = SCardListCards(self.hcontext, dmyATR, [])
                self.assertEqual(hresult, 0)
                self.assertEqual(card, dmyName)
            elif -1 != platform.platform().find('Windows-Vista-6.0'):
                axaltodotnetATR = \
                  [ 0x3B, 0x16, 0x96, 0x41, 0x73, 0x74, 0x72, 0x69, 0x64 ]
                axaltodotnetName = ['Axalto Cryptoflex .NET']
                hresult, card = SCardListCards(self.hcontext, axaltodotnetATR, [])
                self.assertEqual(hresult, 0)
                self.assertEqual(card, axaltodotnetName)
            else:
                slbCryptoFlex8kv2ATR = \
                  [0x3B, 0x95, 0x15, 0x40, 0x00, 0x68, 0x01, 0x02, 0x00, 0x00]
                slbCryptoFlex8kv2Name = ['Schlumberger Cryptoflex 8K v2']
                hresult, card = SCardListCards(
                        self.hcontext, slbCryptoFlex8kv2ATR, [])
                self.assertEqual(hresult, 0)
                self.assertEqual(card, slbCryptoFlex8kv2Name)

        # locate dummy card by interface
        def test_listdummycardbyguid(self):
            guidstolocate = self.dummycardguid2 + self.dummycardguid1
            locatedcardnames = [self.dummycardname]
            hresult, card = SCardListCards(self.hcontext, [], guidstolocate)
            self.assertEqual(hresult, 0)
            self.assertEqual(card, locatedcardnames)

        # list our dummy card interfaces and check
        # that they match the introduced interfaces
        def test_listdummycardinterfaces(self):
            hresult, interfaces = SCardListInterfaces(
                self.hcontext, self.dummycardname)
            self.assertEqual(hresult, 0)
            self.assertEqual(2, len(interfaces))
            self.assertEqual(self.dummycardguid1, interfaces[0])
            self.assertEqual(self.dummycardguid2, interfaces[1])

        # locate all cards and interfaces in the system
        def test_listallcards(self):

            if -1 != platform.platform().find('Windows-7'):
                expectedCards = ['Identity Device (Microsoft Generic Profile)',
                                 'Identity Device (NIST SP 800-73 [PIV])']
            elif -1 != platform.platform().find('Windows-Vista-6.0'):
                expectedCards = [
                    'Axalto Cryptoflex .NET',
                    'Infineon SICRYPT CardModule Card',
                    'dummycard' ]
            else:
                # dummycard has been introduced in the test setup and
                # will be removed in the test teardown. Other cards are
                # the cards present by default on Windows 2000
                expectedCards = ['dummycard', 'GemSAFE Smart Card (8K)',
                    'Schlumberger Cryptoflex 4K',
                    'Schlumberger Cryptoflex 8K',
                    'Schlumberger Cryptoflex 8K v2']
            hresult, cards = SCardListCards(self.hcontext, [], [])
            self.assertEqual(hresult, 0)
            foundCards = {}
            for i in range(len(cards)):
                foundCards[cards[i]] = 1
            for i in expectedCards:
                self.assertTrue(i in foundCards)

            # dummycard has a primary provider,
            # other cards have no primary provider
            if 'Windows-7-6.1.7600' == platform.platform():
                expectedPrimaryProviderResult = {
                        'dummycard': [0, self.dummycardguid1],
                        'Identity Device (Microsoft Generic Profile)': \
                          [2, None],
                        'Identity Device (NIST SP 800-73 [PIV])': \
                          [2, None]}
            else:
                expectedPrimaryProviderResult = {
                        'dummycard': [0, self.dummycardguid1],
                        'GemSAFE': [2, None],
                        'Schlumberger Cryptoflex 4k': [2, None],
                        'Schlumberger Cryptoflex 8k': [2, None],
                        'Schlumberger Cryptoflex 8k v2': [2, None]}
            for i in range(len(cards)):
                hresult, providername = SCardGetCardTypeProviderName(
                    self.hcontext, cards[i], SCARD_PROVIDER_PRIMARY)
                if cards[i] in expectedPrimaryProviderResult:
                    self.assertEqual(
                        hresult,
                        expectedPrimaryProviderResult[cards[i]][0])
                    if hresult == 0:
                        self.assertEqual(
                            providername,
                            smartcard.guid.GUIDToStr(
                                expectedPrimaryProviderResult[cards[i]][1]))

            # dummycard has no CSP, other cards have a CSP
            if 'Windows-7-6.1.7600' == platform.platform():
                expectedProviderCSPResult = {
                        'dummycard': [2, None],
                        'Identity Device (Microsoft Generic Profile)': \
                          [0, 'Microsoft Base Smart Card Crypto Provider'],
                        'Identity Device (NIST SP 800-73 [PIV])': \
                          [0, 'Microsoft Base Smart Card Crypto Provider']}
            else:
                expectedProviderCSPResult = {
                        'dummycard': [2, None],
                        'GemSAFE': [0, 'Gemplus GemSAFE Card CSP v1.0'],
                        'Schlumberger Cryptoflex 4k': \
                          [0, 'Schlumberger Cryptographic Service Provider'],
                        'Schlumberger Cryptoflex 8k': \
                          [0, 'Schlumberger Cryptographic Service Provider'],
                        'Schlumberger Cryptoflex 8k v2': \
                          [0, 'Schlumberger Cryptographic Service Provider']}
            for i in range(len(cards)):
                hresult, providername = SCardGetCardTypeProviderName(
                    self.hcontext, cards[i], SCARD_PROVIDER_CSP)
                if cards[i] in expectedProviderCSPResult:
                    self.assertEqual(
                        hresult, expectedProviderCSPResult[cards[i]][0])
                    self.assertEqual(
                        providername, expectedProviderCSPResult[cards[i]][1])

    def suite():
        suite1 = unittest.makeSuite(testcase_listcards)
        return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



"""Card Names class

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from _bsddb import DBNotFoundError
from bsddb import hashopen
from os import environ
from os.path import join
from pickle import dumps, loads, HIGHEST_PROTOCOL


from smartcard.Synchronization import Synchronization, synchronize
from smartcard.util import toBytes


class __CardNames__(Synchronization):
    """__CardNames__ inner class.

    Stores card names and card types into a bsddb hash database.

    The smartcard.CardNames.CardNames singleton manages the creation
    of the unique instance of this class.
    """

    def __init__(self):
        Synchronization.__init__(self)
        carddb_dir = environ['ALLUSERSPROFILE']
        carddb_file = 'cardnames.bdb'
        carddb_file = join(carddb_dir, carddb_file)
        self.db = hashopen(carddb_file, 'w')

    def __del__(self):
        self.db.sync()
        self.db.close()

    def add(self, cardname, cardtype):
        self.db[cardname] = dumps(cardtype, HIGHEST_PROTOCOL)
        self.db.sync()

    def delete(self, cardname):
        try:
            del self.db[cardname]
        except DBNotFoundError:
            pass

    def dump(self):
        for k, v in list(self.db.items()):
            print(k, repr(loads(v)))

    def find(self, atr, reader=None):
        for k, v in list(self.db.items()):
            if loads(v).matches(atr, reader):
                return k

synchronize(__CardNames__, "add delete dump find")


class CardNames(object):
    """The CardNames organizes cards by a unique name and an associated
    smartcard.CardType.CardType."""

    """The single instance of __CardNames__"""
    instance = None

    def __init__(self):
        """Constructor: create a single instance of __readergroups on
        first call"""
        if CardNames.instance is None:
            CardNames.instance = __CardNames__()

    def __getattr__(self, name):
        """All operators redirected to inner class."""
        return getattr(self.instance, name)


if __name__ == '__main__':
    from smartcard.CardType import ATRCardType

    # define a card by its ATR
    ct = ATRCardType([0x3B, 0x16, 0x94, 0x20, 0x02, 0x01, 0x00, 0x00, 0x0D])

    # create CardName
    cn = CardNames()
    cn.add("Palmera Protect V2", ct)
    cn.dump()
    print(cn.find([0x3B, 0x16, 0x94, 0x20, 0x02, 0x01, 0x00, 0x00, 0x0D]))
    print(cn.find([0x3B, 0x16, 0x94, 0x20, 0x02, 0x01, 0x00, 0x00]))
    cn.delete("Palmera Protect V2")
    print('---------')
    cn.dump()



"""ISO7816-9 error checker.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.sw.ErrorChecker import ErrorChecker
import smartcard.sw.SWExceptions

iso7816_9SW = {
    0x62: (smartcard.sw.SWExceptions.WarningProcessingException,
           {0x82: "End of file/record reached"}),

    0x64: (smartcard.sw.SWExceptions.ExecutionErrorException,
           {0x00: "Execution error"}),

    0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x82: "Security status not satisfied"}),

    0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x80: "Incorrect parameters in data field",
            0x84: "Not enough memory space",
            0x89: "File already exists",
            0x8A: "DF name already exists"}),
}


class ISO7816_9ErrorChecker(ErrorChecker):
    """ISO7816-8 error checker.

    This error checker raises the following exceptions:
      - sw1 sw2
      - 62  82          WarningProcessingException
      - 64  00          ExecutionErrorException
      - 69  82          CheckingErrorException
      - 6A  80,84,89,8A CheckingErrorException

    This checker does not raise exceptions on undefined sw1 values, e.g.:
      - sw1 sw2
      - 63  any
      - 6F  any

    and on undefined sw2 values, e.g.:
      - sw1 sw2
      - 62  81 83
      - 64  any except 00


    Use another checker in the error checking chain, e.g., the
    ISO7816_4SW1ErrorChecker or ISO7816_4ErrorChecker, to raise
    exceptions on these undefined values.
    """

    def __call__(self, data, sw1, sw2):
        """Called to test data, sw1 and sw2 for error.

        @param data:       apdu response data
        @param sw1, sw2:   apdu data status words

        Derived classes must raise a L{smartcard.sw.SWException} upon error."""
        if sw1 in iso7816_9SW:
            exception, sw2dir = iso7816_9SW[sw1]
            if type(sw2dir) == type({}):
                try:
                    message = sw2dir[sw2]
                    raise exception(data, sw1, sw2, message)
                except KeyError:
                    pass


if __name__ == '__main__':
    """Small sample illustrating the use of ISO7816_9ErrorChecker."""
    ecs = ISO7816_9ErrorChecker()
    ecs([], 0x90, 0x00)
    ecs([], 0x6a, 0x81)
    try:
        ecs([], 0x6A, 0x8A)
    except smartcard.sw.SWExceptions.CheckingErrorException as e:
        print(str(e) + " %x %x" % (e.sw1, e.sw2))



"""CardConnectionDecorator that provides exclusive use of a card.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.CardConnectionDecorator import CardConnectionDecorator
from smartcard.Exceptions import CardConnectionException
from smartcard.scard import SCardConnect, SCardDisconnect
from smartcard.scard import SCARD_LEAVE_CARD, SCARD_SHARE_EXCLUSIVE
from smartcard.scard import SCardGetErrorMessage
from smartcard.pcsc import PCSCCardConnection
import smartcard.pcsc


class ExclusiveConnectCardConnection(CardConnectionDecorator):
    '''This decorator uses exclusive access to the card during
    connection to prevent other processes to connect to this card.'''

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def connect(self, protocol=None, mode=None, disposition=None):
        '''Disconnect and reconnect in exclusive mode PCSCCardconnections.'''
        CardConnectionDecorator.connect(self, protocol, mode, disposition)
        component = self.component
        while True:
            if isinstance(component,
                          smartcard.pcsc.PCSCCardConnection.PCSCCardConnection):
                pcscprotocol = PCSCCardConnection.translateprotocolmask(
                    protocol)
                if 0 == pcscprotocol:
                    pcscprotocol = component.getProtocol()

                if component.hcard is not None:
                    hresult = SCardDisconnect(component.hcard,
                                              SCARD_LEAVE_CARD)
                    if hresult != 0:
                        raise CardConnectionException(
                                'Failed to disconnect: ' +
                                SCardGetErrorMessage(hresult))
                hresult, component.hcard, dwActiveProtocol = SCardConnect(
                    component.hcontext, str(component.reader),
                    SCARD_SHARE_EXCLUSIVE, pcscprotocol)
                if hresult != 0:
                    raise CardConnectionException(
                        'Failed to connect with SCARD_SHARE_EXCLUSIVE' +
                        SCardGetErrorMessage(hresult))
                # print('reconnected exclusive')
                break
            if hasattr(component, 'component'):
                component = component.component
            else:
                break



"""PCSC context singleton.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from threading import RLock

from smartcard.scard import *
from smartcard.pcsc.PCSCExceptions import EstablishContextException


class PCSCContext(object):
    """Manage a singleton pcsc context handle."""

    class __PCSCContextSingleton:
        """The actual pcsc context class as a singleton."""

        def __init__(self):
            hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
            if hresult != 0:
                raise EstablishContextException(hresult)

        def getContext(self):
            return self.hcontext

    # the singleton
    mutex = RLock()
    instance = None

    def __init__(self):
        PCSCContext.mutex.acquire()
        try:
            if not PCSCContext.instance:
                PCSCContext.instance = PCSCContext.__PCSCContextSingleton()
        finally:
            PCSCContext.mutex.release()

    def __getattr__(self, name):
        if self.instance:
            return getattr(self.instance, name)



"""ISO7816-8 error checker.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.sw.ErrorChecker import ErrorChecker
import smartcard.sw.SWExceptions

iso7816_8SW = {
    0x63: (smartcard.sw.SWExceptions.WarningProcessingException,
           {0x00: "Authentication failed",
            0xC0: "PIN verification failed. 0 retries before blocking PIN",
            0xC1: "PIN verification failed. 1 retries before blocking PIN",
            0xC2: "PIN verification failed. 2 retries before blocking PIN",
            0xC3: "PIN verification failed. 3 retries before blocking PIN",
            0xC4: "PIN verification failed. 4 retries before blocking PIN",
            0xC5: "PIN verification failed. 5 retries before blocking PIN",
            0xC6: "PIN verification failed. 6 retries before blocking PIN",
            0xC7: "PIN verification failed. 7 retries before blocking PIN",
            0xC8: "PIN verification failed. 8 retries before blocking PIN",
            0xC9: "PIN verification failed. 9 retries before blocking PIN",
            0xCA: "PIN verification failed. 10 retries before blocking PIN",
            0xCB: "PIN verification failed. 11 retries before blocking PIN",
            0xCC: "PIN verification failed. 12 retries before blocking PIN",
            0xCD: "PIN verification failed. 13 retries before blocking PIN",
            0xCE: "PIN verification failed. 14 retries before blocking PIN",
            0xCF: "PIN verification failed. 15 retries before blocking PIN"}),

    0x65: (smartcard.sw.SWExceptions.ExecutionErrorException,
           {0x81: "Memory failure (unsuccessful changing)"}),

    0x66: (smartcard.sw.SWExceptions.SecurityRelatedException,
           {0x00: "The environment cannot be set or modified",
            0x87: "Expected SM data objects missing",
            0x88: "SM data objects incorrect"}),

    0x67: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x00: "Wrong length (emtpy Lc field)"}),

    0x68: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x83: "Final command expected",
            0x84: "Command chaining not supported"}),

    0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x82: "Security status not satisfied",
            0x83: "Authentification method blocked",
            0x84: "Referenced data invalidated",
            0x85: "Conditions of use not satisfied"}),

    0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x81: "Function not supported",
            0x82: "File not found",
            0x86: "Incorrect parameters P1-P2",
            0x88: "Referenced data not found"}),
}


class ISO7816_8ErrorChecker(ErrorChecker):
    """ISO7816-8 error checker.

    This error checker raises the following exceptions:
      - sw1 sw2
      - 63  00,c0-cf    WarningProcessingException
      - 65  81          ExecutionErrorException
      - 66  00,87,88    SecurityRelatedException
      - 67  00          CheckingErrorException
      - 68  82,84       CheckingErrorException
      - 69  82,83,84,85 CheckingErrorException
      - 6A  81,82,86,88 CheckingErrorException

    This checker does not raise exceptions on undefined sw1 values, e.g.:
      - sw1 sw2
      - 62  any
      - 6f  any

    and on undefined sw2 values, e.g.:
      - sw1 sw2
      - 66  81 82
      - 67  any except 00


    Use another checker in the error checking chain, e.g., the
    ISO7816_4SW1ErrorChecker or ISO7816_4ErrorChecker, to raise
    exceptions on these undefined values.
    """

    def __call__(self, data, sw1, sw2):
        """Called to test data, sw1 and sw2 for error.

        @param data:       apdu response data
        @param sw1, sw2:   apdu data status words

        Derived classes must raise a L{smartcard.sw.SWException} upon error."""
        if sw1 in iso7816_8SW:
            exception, sw2dir = iso7816_8SW[sw1]
            if type(sw2dir) == type({}):
                try:
                    message = sw2dir[sw2]
                    raise exception(data, sw1, sw2, message)
                except KeyError:
                    pass


if __name__ == '__main__':
    """Small sample illustrating the use of ISO7816_8ErrorChecker."""
    ecs = ISO7816_8ErrorChecker()
    ecs([], 0x90, 0x00)
    ecs([], 0x6a, 0x83)
    try:
        ecs([], 0x66, 0x87)
    except smartcard.sw.SWExceptions.SecurityRelatedException as e:
        print(str(e) + " %x %x" % (e.sw1, e.sw2))



"""PCSCCardConnection class manages connections thru a PCSC reader.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.CardConnection import CardConnection
from smartcard.Exceptions import (CardConnectionException,
    NoCardException, SmartcardException)

from smartcard.scard import *


def translateprotocolmask(protocol):
    """Translate CardConnection protocol mask into PCSC protocol mask."""
    pcscprotocol = 0
    if None != protocol:
        if CardConnection.T0_protocol & protocol:
            pcscprotocol |= SCARD_PROTOCOL_T0
        if CardConnection.T1_protocol & protocol:
            pcscprotocol |= SCARD_PROTOCOL_T1
        if CardConnection.RAW_protocol & protocol:
            pcscprotocol |= SCARD_PROTOCOL_RAW
        if CardConnection.T15_protocol & protocol:
            pcscprotocol |= SCARD_PROTOCOL_T15
    return pcscprotocol


def translateprotocolheader(protocol):
    """Translate protocol into PCSC protocol header."""
    pcscprotocol = 0
    if None != protocol:
        if CardConnection.T0_protocol == protocol:
            pcscprotocol = SCARD_PCI_T0
        if CardConnection.T1_protocol == protocol:
            pcscprotocol = SCARD_PCI_T1
        if CardConnection.RAW_protocol == protocol:
            pcscprotocol = SCARD_PCI_RAW
    return pcscprotocol

dictProtocolHeader = {SCARD_PCI_T0: 'T0', SCARD_PCI_T1: 'T1',
    SCARD_PCI_RAW: 'RAW'}
dictProtocol = {SCARD_PROTOCOL_T0: 'T0', SCARD_PROTOCOL_T1: 'T1',
    SCARD_PROTOCOL_RAW: 'RAW', SCARD_PROTOCOL_T15: 'T15',
    SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1: 'T0 or T1'}


class PCSCCardConnection(CardConnection):
    """PCSCCard connection class. Handles connection with a card thru a
    PCSC reader."""

    def __init__(self, reader):
        """Construct a new PCSC card connection.

        reader: the reader in which the smartcard to connect to is located.
        """
        CardConnection.__init__(self, reader)
        self.hcard = None
        hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != 0:
            raise CardConnectionException(
                'Failed to establish context : ' + \
                SCardGetErrorMessage(hresult))

    def __del__(self):
        """Destructor. Clean PCSC connection resources."""
        # race condition: module CardConnection
        # can disappear before __del__ is called
        self.disconnect()
        hresult = SCardReleaseContext(self.hcontext)
        if hresult != 0:
            raise CardConnectionException(
                'Failed to release context: ' + \
                SCardGetErrorMessage(hresult))
        CardConnection.__del__(self)

    def connect(self, protocol=None, mode=None, disposition=None):
        """Connect to the card.

        If protocol is not specified, connect with the default
        connection protocol.

        If mode is not specified, connect with SCARD_SHARE_SHARED."""
        CardConnection.connect(self, protocol)
        pcscprotocol = translateprotocolmask(protocol)
        if 0 == pcscprotocol:
            pcscprotocol = self.getProtocol()

        if mode == None:
            mode = SCARD_SHARE_SHARED

        # store the way to dispose the card
        if disposition == None:
            disposition = SCARD_UNPOWER_CARD
        self.disposition = disposition

        hresult, self.hcard, dwActiveProtocol = SCardConnect(
            self.hcontext, str(self.reader), mode, pcscprotocol)
        if hresult != 0:
            self.hcard = None
            if hresult in (SCARD_W_REMOVED_CARD, SCARD_E_NO_SMARTCARD):
                raise NoCardException(
                    'Unable to connect: ' + \
                    SCardGetErrorMessage(hresult))
            else:
                raise CardConnectionException(
                    'Unable to connect with protocol: ' + \
                    dictProtocol[pcscprotocol] + '. ' + \
                    SCardGetErrorMessage(hresult))

        protocol = 0
        if dwActiveProtocol == SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1:
            # special case for T0 | T1
            # this happen when mode=SCARD_SHARE_DIRECT and no protocol is
            # then negociated with the card
            protocol = CardConnection.T0_protocol | CardConnection.T1_protocol
        else:
            for p in dictProtocol:
                if p == dwActiveProtocol:
                    protocol = eval("CardConnection.%s_protocol" % dictProtocol[p])
        PCSCCardConnection.setProtocol(self, protocol)

    def disconnect(self):
        """Disconnect from the card."""

        # when __del__() is invoked in response to a module being deleted,
        # e.g., when execution of the program is done, other globals referenced
        # by the __del__() method may already have been deleted.
        # this causes CardConnection.disconnect to except with a TypeError
        try:
            CardConnection.disconnect(self)
        except TypeError:
            pass
        if None != self.hcard:
            hresult = SCardDisconnect(self.hcard, self.disposition)
            if hresult != 0:
                raise CardConnectionException(
                    'Failed to disconnect: ' + \
                    SCardGetErrorMessage(hresult))
            self.hcard = None

    def getATR(self):
        """Return card ATR"""
        CardConnection.getATR(self)
        if None == self.hcard:
            raise CardConnectionException('Card not connected')
        hresult, reader, state, protocol, atr = SCardStatus(self.hcard)
        if hresult != 0:
            raise CardConnectionException(
                'Failed to get status: ' + \
                SCardGetErrorMessage(hresult))
        return atr

    def doTransmit(self, bytes, protocol=None):
        """Transmit an apdu to the card and return response apdu.

        @param bytes:    command apdu to transmit (list of bytes)

        @param protocol: the transmission protocol, from
            CardConnection.T0_protocol, CardConnection.T1_protocol, or
            CardConnection.RAW_protocol

        @return:     a tuple (response, sw1, sw2) where
                    sw1 is status word 1, e.g. 0x90
                    sw2 is status word 2, e.g. 0x1A
                    response are the response bytes excluding status words
        """
        if None == protocol:
            protocol = self.getProtocol()
        CardConnection.doTransmit(self, bytes, protocol)
        pcscprotocolheader = translateprotocolheader(protocol)
        if 0 == pcscprotocolheader:
            raise CardConnectionException(
                'Invalid protocol in transmit: must be ' + \
                'CardConnection.T0_protocol, ' + \
                'CardConnection.T1_protocol, or ' + \
                'CardConnection.RAW_protocol')
        if None == self.hcard:
            raise CardConnectionException('Card not connected')
        hresult, response = SCardTransmit(
            self.hcard, pcscprotocolheader, bytes)
        if hresult != 0:
            raise CardConnectionException(
                'Failed to transmit with protocol ' + \
                dictProtocolHeader[pcscprotocolheader] + '. ' + \
                SCardGetErrorMessage(hresult))

        sw1 = (response[-2] + 256) % 256
        sw2 = (response[-1] + 256) % 256

        data = [(x + 256) % 256 for x in response[:-2]]
        return list(data), sw1, sw2

    def doControl(self, controlCode, bytes=[]):
        """Transmit a control command to the reader and return response.

        controlCode: control command

        bytes:       command data to transmit (list of bytes)

        return:      response are the response bytes (if any)
        """
        CardConnection.doControl(self, controlCode, bytes)
        hresult, response = SCardControl(self.hcard, controlCode, bytes)
        if hresult != 0:
            raise SmartcardException(
                'Failed to control ' + SCardGetErrorMessage(hresult))

        data = [(x + 256) % 256 for x in response]
        return list(data)

    def doGetAttrib(self, attribId):
        """get an attribute

        attribId: Identifier for the attribute to get

        return:   response are the attribute byte array
        """
        CardConnection.doGetAttrib(self, attribId)
        hresult, response = SCardGetAttrib(self.hcard, attribId)
        if hresult != 0:
            raise SmartcardException(
                'Failed to getAttrib ' + SCardGetErrorMessage(hresult))
        return response


if __name__ == '__main__':
    """Small sample illustrating the use of CardConnection."""
    SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
    DF_TELECOM = [0x7F, 0x10]
    from smartcard.pcsc.PCSCReader import PCSCReader
    from smartcard.pcsc.PCSCPart10 import CM_IOCTL_GET_FEATURE_REQUEST
    cc = PCSCReader.readers()[0].createConnection()
    cc.connect()
    print("%r %x %x" % cc.transmit(SELECT + DF_TELECOM))

    print(cc.control(CM_IOCTL_GET_FEATURE_REQUEST, []))




"""wxPython panel display cards/readers as a TreeCtrl.

__author__ = "gemalto http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

# smartcard imports
from threading import RLock
from smartcard.Exceptions import CardConnectionException, NoCardException
from smartcard.ReaderMonitoring import ReaderMonitor, ReaderObserver
from smartcard.CardMonitoring import CardMonitor, CardObserver
from smartcard.util import toHexString

import smartcard.wx.SimpleSCardApp
from smartcard.wx import ICO_SMARTCARD, ICO_READER

# wxPython GUI modules (http://www.wxpython.org)
import wx


class BaseCardTreeCtrl(wx.TreeCtrl):
    """Base class for the smart card and reader tree controls."""

    def __init__(self,
                  parent,
                  ID=wx.NewId(),
                  pos=wx.DefaultPosition,
                  size=wx.DefaultSize,
                  style=0,
                  clientpanel=None):
        """Constructor. Initializes a smartcard or reader tree control."""
        wx.TreeCtrl.__init__(
            self, parent, ID, pos, size, wx.TR_SINGLE | wx.TR_NO_BUTTONS)

        self.clientpanel = clientpanel
        self.parent = parent

        isz = (16, 16)
        il = wx.ImageList(isz[0], isz[1])
        self.capindex = il.Add(
            wx.ArtProvider_GetBitmap(wx.ART_HELP_BOOK, wx.ART_OTHER, isz))
        self.fldrindex = il.Add(
            wx.ArtProvider_GetBitmap(wx.ART_FOLDER, wx.ART_OTHER, isz))
        self.fldropenindex = il.Add(
            wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN, wx.ART_OTHER, isz))
        if None != ICO_SMARTCARD:
            self.cardimageindex = il.Add(
                wx.Bitmap(ICO_SMARTCARD, wx.BITMAP_TYPE_ICO))
        if None != ICO_READER:
            self.readerimageindex = il.Add(
                wx.Bitmap(ICO_READER, wx.BITMAP_TYPE_ICO))
        self.il = il
        self.SetImageList(self.il)

    def Repaint(self):
        self.Refresh()
        wx.PostEvent(self, wx.PaintEvent())


class CardTreeCtrl(BaseCardTreeCtrl):
    """The CardTreeCtrl monitors inserted cards and notifies the
    application client dialog of any card activation."""

    def __init__(self,
                  parent,
                  ID=wx.NewId(),
                  pos=wx.DefaultPosition,
                  size=wx.DefaultSize,
                  style=0,
                  clientpanel=None):
        """Constructor. Create a smartcard tree control."""
        BaseCardTreeCtrl.__init__(self, parent, ID, pos, size,
                                   wx.TR_SINGLE | wx.TR_NO_BUTTONS,
                                   clientpanel)

        self.root = self.AddRoot("Smartcards")
        self.SetPyData(self.root, None)
        self.SetItemImage(self.root, self.fldrindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(
            self.root, self.fldropenindex, wx.TreeItemIcon_Expanded)
        self.Expand(self.root)

    def OnAddCards(self, addedcards):
        """Called when a card is inserted.
        Adds a smart card to the smartcards tree."""
        parentnode = self.root
        for cardtoadd in addedcards:
            childCard = self.AppendItem(parentnode, toHexString(cardtoadd.atr))
            self.SetItemText(childCard, toHexString(cardtoadd.atr))
            self.SetPyData(childCard, cardtoadd)
            self.SetItemImage(
                childCard, self.cardimageindex, wx.TreeItemIcon_Normal)
            self.SetItemImage(
                childCard, self.cardimageindex, wx.TreeItemIcon_Expanded)
            self.Expand(childCard)
        self.Expand(self.root)
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveCards(self, removedcards):
        """Called when a card is removed.
        Removes a card from the tree."""
        parentnode = self.root
        for cardtoremove in removedcards:
            (childCard, cookie) = self.GetFirstChild(parentnode)
            while childCard.IsOk():
                if self.GetItemText(childCard) == \
                        toHexString(cardtoremove.atr):
                    self.Delete(childCard)
                (childCard, cookie) = self.GetNextChild(parentnode, cookie)
        self.Expand(self.root)
        self.EnsureVisible(self.root)
        self.Repaint()


class ReaderTreeCtrl(BaseCardTreeCtrl):
    """The ReaderTreeCtrl monitors inserted cards and readers and notifies the
    application client dialog of any card activation."""

    def __init__(self,
                  parent,
                  ID=wx.NewId(),
                  pos=wx.DefaultPosition,
                  size=wx.DefaultSize,
                  style=0,
                  clientpanel=None):
        """Constructor. Create a reader tree control."""

        BaseCardTreeCtrl.__init__(
            self,
            parent,
            ID,
            pos,
            size,
            wx.TR_SINGLE | wx.TR_NO_BUTTONS,
            clientpanel)

        self.mutex = RLock()

        self.root = self.AddRoot("Smartcard Readers")
        self.SetPyData(self.root, None)
        self.SetItemImage(
            self.root, self.fldrindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(
            self.root, self.fldropenindex, wx.TreeItemIcon_Expanded)
        self.Expand(self.root)

    def AddATR(self, readernode, atr):
        """Add an ATR to a reader node."""
        capchild = self.AppendItem(readernode, atr)
        self.SetPyData(capchild, None)
        self.SetItemImage(
            capchild, self.cardimageindex, wx.TreeItemIcon_Normal)
        self.SetItemImage(
            capchild, self.cardimageindex, wx.TreeItemIcon_Expanded)
        self.Expand(capchild)
        return capchild

    def GetATR(self, reader):
        """Return the ATR of the card inserted into the reader."""
        atr = "no card inserted"
        try:
            if not type(reader) is str:
                connection = reader.createConnection()
                connection.connect()
                atr = toHexString(connection.getATR())
                connection.disconnect()
        except NoCardException:
            pass
        except CardConnectionException:
            pass
        return atr

    def OnAddCards(self, addedcards):
        """Called when a card is inserted.
        Adds the smart card child to the reader node."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for cardtoadd in addedcards:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                found = False
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == str(cardtoadd.reader):
                        (childCard, cookie2) = self.GetFirstChild(childReader)
                        self.SetItemText(childCard, toHexString(cardtoadd.atr))
                        self.SetPyData(childCard, cardtoadd)
                        found = True
                    else:
                        (childReader, cookie) = self.GetNextChild(
                            parentnode, cookie)

                # reader was not found, add reader node
                # this happens when card monitoring thread signals
                # added cards before reader monitoring thread signals
                # added readers
                if not found:
                    childReader = self.AppendItem(
                        parentnode, str(cardtoadd.reader))
                    self.SetPyData(childReader, cardtoadd.reader)
                    self.SetItemImage(
                        childReader,
                        self.readerimageindex,
                        wx.TreeItemIcon_Normal)
                    self.SetItemImage(
                        childReader,
                        self.readerimageindex,
                        wx.TreeItemIcon_Expanded)
                    childCard = self.AddATR(
                        childReader, toHexString(cardtoadd.atr))
                    self.SetPyData(childCard, cardtoadd)
                    self.Expand(childReader)

            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnAddReaders(self, addedreaders):
        """Called when a reader is inserted.
        Adds the smart card reader to the smartcard readers tree."""
        self.mutex.acquire()

        try:
            parentnode = self.root
            for readertoadd in addedreaders:
                # is the reader already here?
                found = False
                (childReader, cookie) = self.GetFirstChild(parentnode)
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == str(readertoadd):
                        found = True
                    else:
                        (childReader, cookie) = self.GetNextChild(
                                                    parentnode, cookie)
                if not found:
                    childReader = self.AppendItem(parentnode, str(readertoadd))
                    self.SetPyData(childReader, readertoadd)
                    self.SetItemImage(
                        childReader,
                        self.readerimageindex,
                        wx.TreeItemIcon_Normal)
                    self.SetItemImage(
                        childReader,
                        self.readerimageindex,
                        wx.TreeItemIcon_Expanded)
                    self.AddATR(
                        childReader,
                        self.GetATR(readertoadd))
                    self.Expand(childReader)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveCards(self, removedcards):
        """Called when a card is removed.
        Removes the card from the tree."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for cardtoremove in removedcards:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                found = False
                while childReader.IsOk() and not found:
                    if self.GetItemText(childReader) == \
                            str(cardtoremove.reader):
                        (childCard, cookie2) = self.GetFirstChild(childReader)
                        self.SetItemText(childCard, 'no card inserted')
                        found = True
                    else:
                        (childReader, cookie) = \
                            self.GetNextChild(parentnode, cookie)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()

    def OnRemoveReaders(self, removedreaders):
        """Called when a reader is removed.
        Removes the reader from the smartcard readers tree."""
        self.mutex.acquire()
        try:
            parentnode = self.root
            for readertoremove in removedreaders:
                (childReader, cookie) = self.GetFirstChild(parentnode)
                while childReader.IsOk():
                    if self.GetItemText(childReader) == str(readertoremove):
                        self.Delete(childReader)
                    else:
                        (childReader, cookie) = \
                            self.GetNextChild(parentnode, cookie)
            self.Expand(self.root)
        finally:
            self.mutex.release()
        self.EnsureVisible(self.root)
        self.Repaint()


class CardAndReaderTreePanel(wx.Panel):
    """Panel containing the smart card and reader tree controls."""

    class _CardObserver(CardObserver):
        """Inner CardObserver. Gets notified of card insertion
        removal by the CardMonitor."""

        def __init__(self, cardtreectrl):
            self.cardtreectrl = cardtreectrl

        def update(self, observable, handlers):
            """CardObserver callback that is notified
            when cards are added or removed."""
            addedcards, removedcards = handlers
            self.cardtreectrl.OnRemoveCards(removedcards)
            self.cardtreectrl.OnAddCards(addedcards)

    class _ReaderObserver(ReaderObserver):
        """Inner ReaderObserver. Gets notified of reader insertion/removal
        by the ReaderMonitor."""

        def __init__(self, readertreectrl):
            self.readertreectrl = readertreectrl

        def update(self, observable, handlers):
            """ReaderObserver callback that is notified when
            readers are added or removed."""
            addedcards, removedcards = handlers
            self.readertreectrl.OnRemoveReaders(removedreaders)
            self.readertreectrl.OnAddReaders(addedreaders)

    def __init__(self, parent, appstyle, clientpanel):
        """Constructor. Create a smartcard and reader tree control on the
        left-hand side of the application main frame.
        @param parent: the tree panel parent
        @param appstyle: a combination of the following styles (bitwise or |)
          - TR_SMARTCARD: display a smartcard tree panel
          - TR_READER: display a reader tree panel
          - default is TR_DEFAULT = TR_SMARTCARD
        @param clientpanel: the client panel to notify of smartcard and reader events
        """
        wx.Panel.__init__(self, parent, -1, style=wx.WANTS_CHARS)

        sizer = wx.BoxSizer(wx.VERTICAL)

        # create the smartcard tree
        if appstyle & smartcard.wx.SimpleSCardApp.TR_SMARTCARD:
            self.cardtreectrl = CardTreeCtrl(self, clientpanel=clientpanel)

            # create the smartcard insertion observer
            self.cardtreecardobserver = self._CardObserver(self.cardtreectrl)

            # register as a CardObserver; we will ge
            # notified of added/removed cards
            self.cardmonitor = CardMonitor()
            self.cardmonitor.addObserver(self.cardtreecardobserver)

            sizer.Add(
                self.cardtreectrl, flag=wx.EXPAND | wx.ALL, proportion=1)

        # create the reader tree
        if appstyle & smartcard.wx.SimpleSCardApp.TR_READER:
            self.readertreectrl = ReaderTreeCtrl(
                                        self, clientpanel=clientpanel)

            # create the reader insertion observer
            self.readertreereaderobserver = self._ReaderObserver(
                                                    self.readertreectrl)

            # register as a ReaderObserver; we will ge
            # notified of added/removed readers
            self.readermonitor = ReaderMonitor()
            self.readermonitor.addObserver(self.readertreereaderobserver)

            # create the smartcard insertion observer
            self.readertreecardobserver = self._CardObserver(
                                                self.readertreectrl)

            # register as a CardObserver; we will get
            # notified of added/removed cards
            self.cardmonitor = CardMonitor()
            self.cardmonitor.addObserver(self.readertreecardobserver)

            sizer.Add(
                self.readertreectrl, flag=wx.EXPAND | wx.ALL, proportion=1)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)

    def OnDestroy(self, event):
        """Called on panel destruction."""
        # deregister observers
        if hasattr(self, 'cardmonitor'):
            self.cardmonitor.deleteObserver(self.cardtreecardobserver)
        if hasattr(self, 'readermonitor'):
            self.readermonitor.deleteObserver(self.readertreereaderobserver)
            self.cardmonitor.deleteObserver(self.readertreecardobserver)
        event.Skip()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import unittest
from smartcard.ATR import ATR
from smartcard.Exceptions import SmartcardException
from smartcard.util import toBytes

if sys.version_info >= (3, 0):
    from io import StringIO
else:
    from StringIO import StringIO


class TestUtil(unittest.TestCase):

    def setUp(self):
        self.held, sys.stdout = sys.stdout, StringIO()

    def test_ATR1(self):
        atr = [0x3F, 0x65, 0x25, 0x00, 0x2C, 0x09, 0x69, 0x90, 0x00]
        data_out = """TB1: 25
TC1: 0
supported protocols T=0
T=0 supported: True
T=1 supported: False
	clock rate conversion factor: 372
	bit rate adjustment factor: 1
	maximum programming current: 50
	programming voltage: 30
	guard time: 0
nb of interface bytes: 2
nb of historical bytes: 5
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR2(self):
        atr = [0x3F, 0x65, 0x25, 0x08, 0x93, 0x04, 0x6C, 0x90, 0x00]
        data_out = """TB1: 25
TC1: 8
supported protocols T=0
T=0 supported: True
T=1 supported: False
	clock rate conversion factor: 372
	bit rate adjustment factor: 1
	maximum programming current: 50
	programming voltage: 30
	guard time: 8
nb of interface bytes: 2
nb of historical bytes: 5
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR3(self):
        atr = [0x3B, 0x16, 0x94, 0x7C, 0x03, 0x01, 0x00, 0x00, 0x0D]
        data_out = """TA1: 94
supported protocols T=0
T=0 supported: True
T=1 supported: False
	clock rate conversion factor: 512
	bit rate adjustment factor: 8
	maximum programming current: 50
	programming voltage: 5
	guard time: None
nb of interface bytes: 1
nb of historical bytes: 6
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR4(self):
        atr = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
        data_out = """TB1: 0
TC1: 0
supported protocols T=0
T=0 supported: True
T=1 supported: False
	clock rate conversion factor: 372
	bit rate adjustment factor: 1
	maximum programming current: 25
	programming voltage: 5
	guard time: 0
nb of interface bytes: 2
nb of historical bytes: 5
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR5(self):
        atr = [0x3B, 0xE3, 0x00, 0xFF, 0x81, 0x31, 0x52, 0x45, 0xA1,
               0xA2, 0xA3, 0x1B]
        data_out = """TB1: 0
TC1: ff
TD1: 81
TD2: 31
TA3: 52
TB3: 45
supported protocols T=1
T=0 supported: False
T=1 supported: True
checksum: 27
	clock rate conversion factor: 372
	bit rate adjustment factor: 1
	maximum programming current: 25
	programming voltage: 5
	guard time: 255
nb of interface bytes: 6
nb of historical bytes: 3
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR6(self):
        atr = [0x3B, 0xE5, 0x00, 0x00, 0x81, 0x21, 0x45, 0x9C, 0x10,
               0x01, 0x00, 0x80, 0x0D]
        data_out = """TB1: 0
TC1: 0
TD1: 81
TD2: 21
TB3: 45
supported protocols T=1
T=0 supported: False
T=1 supported: True
checksum: 13
	clock rate conversion factor: 372
	bit rate adjustment factor: 1
	maximum programming current: 25
	programming voltage: 5
	guard time: 0
nb of interface bytes: 5
nb of historical bytes: 5
"""
        a = ATR(atr)
        a.dump()
        output = sys.stdout.getvalue()
        self.assertEqual(output, data_out)

    def test_ATR_TS(self):
        atr = [0x42]
        self.assertRaises(SmartcardException, ATR, atr)

    def test_ATR_get(self):
        atr = "3B F2 95 12 34 01 36 06"
        a = ATR(toBytes(atr))
        self.assertEqual(a.getTA1(), 0x95)
        self.assertEqual(a.getTB1(), 0x12)
        self.assertEqual(a.getTC1(), 0x34)
        self.assertEqual(a.getTD1(), 0x01)
        self.assertEqual(a.getHistoricalBytes(), [0x36, 0x06])
        self.assertFalse(a.isT15Supported())
        self.assertEqual(str(a), atr)

if __name__ == '__main__':
    unittest.main(buffer=True)



"""smartcard.guid

Utility functions to handle GUIDs as strings or list of bytes

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
import uuid

# guid is ulong+ushort+ushort+uchar[8]


def strToGUID(s):
    """Converts a GUID string into a list of bytes.

    >>> strToGUID('{AD4F1667-EA75-4124-84D4-641B3B197C65}')
    [103, 22, 79, 173, 117, 234, 36, 65, 132, 212, 100, 27, 59, 25, 124, 101]
    """
    dat = uuid.UUID(hex=s)
    if isinstance(dat.bytes_le, str):
        # Python 2
        dat = [ord(e) for e in dat.bytes_le]
    else:
        # Python 3
        dat = list(dat.bytes_le)
    return dat


def GUIDToStr(g):
    """Converts a GUID sequence of bytes into a string.

    >>> GUIDToStr([103,22,79,173,  117,234,  36,65,
    ...            132, 212, 100, 27, 59, 25, 124, 101])
    '{AD4F1667-EA75-4124-84D4-641B3B197C65}'
    """
    try:
        dat = uuid.UUID(bytes_le=bytes(g))
    except:
        dat = uuid.UUID(bytes_le=''.join(map(chr, g)))
    return '{' + str(dat).upper() + '}'


if __name__ == "__main__":
    """Small sample illustrating the use of guid.py."""
    guid_in = '{AD4F1667-EA75-4124-84D4-641B3B197C65}'
    print(guid_in)
    dummycardguid1 = strToGUID(guid_in)
    print(dummycardguid1)
    guid_out = GUIDToStr(dummycardguid1)
    print(guid_out)
    if guid_in != guid_out:
        print("Failure")
    else:
        print("Success")



"""Sample CardConnectionDecorator that provides exclusive transmit()

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.CardConnectionDecorator import CardConnectionDecorator
from smartcard.Exceptions import CardConnectionException
from smartcard.scard import SCardBeginTransaction, SCardEndTransaction
from smartcard.scard import SCARD_LEAVE_CARD
from smartcard.scard import SCardGetErrorMessage
import smartcard.pcsc


class ExclusiveTransmitCardConnection(CardConnectionDecorator):
    '''This decorator uses SCardBeginTransaction/SCardEndTransaction to
    preserve other processes of threads to access the card during
    transmit().'''

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def lock(self):
        '''Lock card with SCardBeginTransaction.'''

        component = self.component
        while True:
            if isinstance(
                    component,
                    smartcard.pcsc.PCSCCardConnection.PCSCCardConnection):
                hresult = SCardBeginTransaction(component.hcard)
                if 0 != hresult:
                    raise CardConnectionException(
                        'Failed to lock with SCardBeginTransaction: ' +
                        SCardGetErrorMessage(hresult))
                else:
                    # print('locked')
                    pass
                break
            if hasattr(component, 'component'):
                component = component.component
            else:
                break

    def unlock(self):
        '''Unlock card with SCardEndTransaction.'''
        component = self.component
        while True:
            if isinstance(
                    component,
                    smartcard.pcsc.PCSCCardConnection.PCSCCardConnection):
                hresult = SCardEndTransaction(component.hcard,
                                              SCARD_LEAVE_CARD)
                if 0 != hresult:
                    raise CardConnectionException(
                        'Failed to unlock with SCardEndTransaction: ' +
                        SCardGetErrorMessage(hresult))
                else:
                    # print('unlocked')
                    pass
                break
            if hasattr(component, 'component'):
                component = component.component
            else:
                break

    def transmit(self, bytes, protocol=None):
        '''Gain exclusive access to card during APDU transmission for if this
        decorator decorates a PCSCCardConnection.'''
        data, sw1, sw2 = CardConnectionDecorator.transmit(
            self, bytes, protocol)
        return data, sw1, sw2



#! /usr/bin/env python
"""Unit tests for smartcard.Card

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
import unittest
import string

# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

try:
    from local_config import expectedATRs, expectedReaders
    from local_config import expectedReaderGroups, expectedATRinReader
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


from smartcard.Exceptions import NoCardException
from smartcard.Card import Card
from smartcard.System import readers


class testcase_Card(unittest.TestCase):
    """Test case for smartcard.Card."""

    def testcase_CardDictionary(self):
        """Create a dictionnary with Card keys"""
        mydict = {}
        for reader in readers():
            card = Card(reader, expectedATRinReader[str(reader)])
            mydict[card] = reader

        for card in list(mydict.keys()):
            self.assertTrue(str(card.reader) in expectedReaders)

    def testcase_Card_FromReaders(self):
        """Create a Card from Readers and test that the response
        to SELECT DF_TELECOM has two bytes."""
        SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
        DF_TELECOM = [0x7F, 0x10]

        for reader in readers():
            card = Card(reader, expectedATRinReader[str(reader)])
            cc = card.createConnection()
            if [] != expectedATRinReader[str(reader)]:
                cc.connect()
                response, sw1, sw2 = cc.transmit(SELECT + DF_TELECOM)
                expectedSWs = {"9f 1a": 1, "9f 20": 2, "6e 0": 3}
                self.assertEqual([], response)
                self.assertTrue(
                    "%x %x" % (sw1, sw2) in expectedSWs or "9f" == "%x" % sw1)
            else:
                self.assertRaises(NoCardException, cc.connect)

    def testcase_Card_FromReaderStrings(self):
        """Create a Card from reader strings and test that the response
        to SELECT DF_TELECOM has two bytes."""
        SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
        DF_TELECOM = [0x7F, 0x10]

        for reader in readers():
            card = Card(str(reader), expectedATRinReader[str(reader)])
            cc = card.createConnection()
            if [] != expectedATRinReader[str(reader)]:
                cc.connect()
                response, sw1, sw2 = cc.transmit(SELECT + DF_TELECOM)
                expectedSWs = {"9f 1a": 1, "9f 20": 2, "6e 0": 3}
                self.assertEqual([], response)
                self.assertTrue(
                    "%x %x" % (sw1, sw2) in expectedSWs or "9f" == "%x" % sw1)
            else:
                self.assertRaises(NoCardException, cc.connect)

    def testcase_Card_Eq_NotEq(self):
        """Test == and != for Cards."""
        for reader in readers():
            card = Card(str(reader), expectedATRinReader[str(reader)])
            cardcopy = Card(str(reader), expectedATRinReader[str(reader)])
            self.assertEqual(True, card == cardcopy)
            self.assertEqual(True, not card != cardcopy)

        for reader in readers():
            card = Card(str(reader), expectedATRinReader[str(reader)])
            cardcopy = Card(str(reader), [0, 0])
            self.assertEqual(True, card != cardcopy)
            self.assertEqual(True, not card == cardcopy)


def suite():
    suite1 = unittest.makeSuite(testcase_CardConnection)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for SCardGetErrorMessage.

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import unittest
from smartcard.scard import *
import sys


class testcase_geterrormessage(unittest.TestCase):
    """Test scard API for ATR retrieval with SCardLocateCards"""

    def setUp(self):
        hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        self.assertEqual(hresult, 0)

    def tearDown(self):
        hresult = SCardReleaseContext(self.hcontext)
        self.assertEqual(hresult, 0)

    def test_getErrorMessage(self):
        hresult, readers = SCardListReaders(self.hcontext, [])
        self.assertEqual(hresult, 0)

        # we need a long on Python 2.x like 123L
        # but Python 3 do not support the 123L notation any more
        hresult = SCardReleaseContext(pow(2,63) >> 60)
        if 'win32' == sys.platform:
            self.assertEqual((
                SCARD_E_INVALID_HANDLE == hresult or \
                ERROR_INVALID_HANDLE == hresult),
                True)
        else:
            self.assertEqual((SCARD_E_INVALID_HANDLE == hresult), True)
        self.assertEqual((
            SCardGetErrorMessage(hresult).rstrip() == \
                'Invalid handle.'.rstrip() or \
            SCardGetErrorMessage(hresult).rstrip() == \
                'The handle is invalid.'.rstrip()),
            True)


def suite():
    suite1 = unittest.makeSuite(testcase_geterrormessage)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for return codes

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "http://www.gemalto.com"

Copyright 2009 gemalto
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import unittest
from smartcard.scard import *


class testcase_returncodes(unittest.TestCase):
    """Test scard API for return codes """

    def test_getReturnCodes(self):
        errors = (
            SCARD_S_SUCCESS,
            SCARD_F_INTERNAL_ERROR,
            SCARD_E_CANCELLED,
            SCARD_E_INVALID_HANDLE,
            SCARD_E_INVALID_PARAMETER,
            SCARD_E_INVALID_TARGET,
            SCARD_E_NO_MEMORY,
            SCARD_F_WAITED_TOO_LONG,
            SCARD_E_INSUFFICIENT_BUFFER,
            SCARD_E_UNKNOWN_READER,
            SCARD_E_TIMEOUT,
            SCARD_E_SHARING_VIOLATION,
            SCARD_E_NO_SMARTCARD,
            SCARD_E_UNKNOWN_CARD,
            SCARD_E_CANT_DISPOSE,
            SCARD_E_PROTO_MISMATCH,
            SCARD_E_NOT_READY,
            SCARD_E_INVALID_VALUE,
            SCARD_E_SYSTEM_CANCELLED,
            SCARD_F_COMM_ERROR,
            SCARD_F_UNKNOWN_ERROR,
            SCARD_E_INVALID_ATR,
            SCARD_E_NOT_TRANSACTED,
            SCARD_E_READER_UNAVAILABLE,
            SCARD_E_PCI_TOO_SMALL,
            SCARD_E_READER_UNSUPPORTED,
            SCARD_E_DUPLICATE_READER,
            SCARD_E_CARD_UNSUPPORTED,
            SCARD_E_NO_SERVICE,
            SCARD_E_SERVICE_STOPPED,
            SCARD_E_UNEXPECTED,
            SCARD_E_ICC_INSTALLATION,
            SCARD_E_ICC_CREATEORDER,
            SCARD_E_UNSUPPORTED_FEATURE,
            SCARD_E_DIR_NOT_FOUND,
            SCARD_E_FILE_NOT_FOUND,
            SCARD_E_NO_DIR,
            SCARD_E_NO_FILE,
            SCARD_E_NO_ACCESS,
            SCARD_E_WRITE_TOO_MANY,
            SCARD_E_BAD_SEEK,
            SCARD_E_INVALID_CHV,
            SCARD_E_UNKNOWN_RES_MNG,
            SCARD_E_NO_SUCH_CERTIFICATE,
            SCARD_E_CERTIFICATE_UNAVAILABLE,
            SCARD_E_NO_READERS_AVAILABLE,
            SCARD_E_COMM_DATA_LOST,
            SCARD_E_NO_KEY_CONTAINER,
            SCARD_E_SERVER_TOO_BUSY,
            SCARD_W_UNSUPPORTED_CARD,
            SCARD_W_UNRESPONSIVE_CARD,
            SCARD_W_UNPOWERED_CARD,
            SCARD_W_RESET_CARD,
            SCARD_W_REMOVED_CARD,
            SCARD_W_SECURITY_VIOLATION,
            SCARD_W_WRONG_CHV,
            SCARD_W_CHV_BLOCKED,
            SCARD_W_EOF,
            SCARD_W_CANCELLED_BY_USER,
            SCARD_W_CARD_NOT_AUTHENTICATED,
            )
        #for e in errors:
        #    print(hex((e+0x100000000) & 0xFFFFFFFF), SCardGetErrorMessage(e))


def suite():
    suite1 = unittest.makeSuite(testcase_returncodes)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



"""Smart card Reader abstract class.

__author__ = "gemalto http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


class Reader(object):
    """Reader abstract class.

    The reader class is responsible for creating connections
    with a card.

    Known subclasses: PCSCReader
    """

    def __init__(self, readername):
        """Constructs a new reader and store readername."""
        self.name = readername

    def addtoreadergroup(self, groupname):
        """Add reader to a reader group."""
        pass

    def removefromreadergroup(self, groupname):
        """Remove reader from a reader group."""
        pass

    def createConnection(self):
        """Returns a card connection thru reader."""
        pass

    def __eq__(self, other):
        """Returns True if self==other (same name)."""
        if type(other) == type(self):
            return self.name == other.name
        else:
            return False

    def __hash__(self):
        """Returns a hash value for this object (self.name is unique)."""
        return hash(self.name)

    def __repr__(self):
        """Returns card reader name string for `object` calls."""
        return "'%s'" % self.name

    def __str__(self):
        """Returns card reader name string for str(object) calls."""
        return self.name



#! /usr/bin/env python
"""
Sample script to illustrate toHexString() utility method

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.util import *


print(40 * '-')
bytes = [59, 101, 0, 0, 156, 17, 1, 1, 3]
print('bytes = [59, 101, 0, 0, 156, 17, 1, 1, 3]')
print('toHexString(bytes) =', toHexString(bytes))
print('toHexString(bytes, COMMA) =', toHexString(bytes, COMMA))
print('toHexString(bytes, PACK) =', toHexString(bytes, PACK))
print('toHexString(bytes, HEX) =', toHexString(bytes, HEX))
print('toHexString(bytes, HEX | COMMA) =', toHexString(bytes, HEX | COMMA))
print('toHexString(bytes, HEX | UPPERCASE) =',
      toHexString(bytes, HEX | UPPERCASE))
print('toHexString(bytes, HEX | UPPERCASE | COMMA) =',
      toHexString(bytes, HEX | UPPERCASE | COMMA))


print(40 * '-')
bytes = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
print('bytes = [ 0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03 ]')
print('toHexString(bytes, COMMA) =', toHexString(bytes, COMMA))
print('toHexString(bytes) =', toHexString(bytes))
print('toHexString(bytes, PACK) =', toHexString(bytes, PACK))
print('toHexString(bytes, HEX) =', toHexString(bytes, HEX))
print('toHexString(bytes, HEX | COMMA) =', toHexString(bytes, HEX | COMMA))
print('toHexString(bytes, HEX | UPPERCASE) =',
      toHexString(bytes, HEX | UPPERCASE))
print('toHexString(bytes, HEX | UPPERCASE | COMMA) =',
      toHexString(bytes, HEX | UPPERCASE | COMMA))


import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
"""
Sample script that displays the ATR of inserted cards.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.Exceptions import NoCardException
from smartcard.System import readers
from smartcard.util import toHexString
import sys

# Use a connection context
# the connection object is automatically destroyed at the end of with block

for reader in readers():
    with reader.createConnection() as connection:
        try:
            connection.connect()
            print(reader, toHexString(connection.getATR()))
        except NoCardException:
            print(reader, 'no card inserted')

# sleep(5)

if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



"""ClassLoader allows you to load modules from packages without
hard-coding their class names in code; instead, they might be
specified in a configuration file, as command-line parameters,
or within an interface.

Source: Robert Brewer at the Python Cookbook:
http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/223972

License: PSF license (http://docs.python.org/license.html).
"""


def get_mod(modulePath):
    """Import a module."""
    return __import__(modulePath, globals(), locals(), [''])


def get_func(fullFuncName):
    """Retrieve a function object from a full dotted-package name."""

    # Parse out the path, module, and function
    lastDot = fullFuncName.rfind(u".")
    funcName = fullFuncName[lastDot + 1:]
    modPath = fullFuncName[:lastDot]

    aMod = get_mod(modPath)
    aFunc = getattr(aMod, funcName)

    # Assert that the function is a *callable* attribute.
    assert callable(aFunc), u"%s is not callable." % fullFuncName

    # Return a reference to the function itself,
    # not the results of the function.
    return aFunc


def get_class(fullClassName, parentClass=None):
    """Load a module and retrieve a class (NOT an instance).

    If the parentClass is supplied, className must be of parentClass
    or a subclass of parentClass (or None is returned).
    """
    aClass = get_func(fullClassName)

    # Assert that the class is a subclass of parentClass.
    if parentClass is not None:
        if not issubclass(aClass, parentClass):
            raise TypeError(u"%s is not a subclass of %s" %
                            (fullClassName, parentClass))

    # Return a reference to the class itself, not an instantiated object.
    return aClass
